/**********************************************************************
 *                                                                    *
 *                              demin3s                               *
 *                                                                    *
 *                A simple cli based minesweeper game                 *
 *                      gcc -o demin3s demin3s.c                      *
 *                                                                    *
 *    Instructions: Digit a number between zero and eight and try     *
 *              to uncover all the cell expected mines one!           *
 *                                                                    *
 *       This game is released as open source under GPL license       *
 *        http://creativecommons.org/licenses/GPL/2.0/deed.it         *
 *             http://www.gnu.org/licenses/gpl-2.0.txt                *
 *                                                                    *
 *   This game is provided without warranty, use it at your own risk! *
 *                                                                    *
 *                     pix3lworkshop.altervista.org                   *
 *                                                                    *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define SIZE 8
#define MAX_MINES 10

int map[SIZE][SIZE], i, j, x, y, mines, minefree;
char playfield[SIZE][SIZE];

  // Draw the playfield
  void draw()
   {
	printf("   0  1  2  3  4  5  6  7\n  +---+---+---+---+---+---+");
	for(i=0; i<SIZE; i++)
     {
      printf("\n%d ", i);
      for(j=0; j<SIZE; j++)
	   printf(" %c ", playfield[i][j]);
     }
    printf("\n  +---+---+---+---+---+---+\n");
   }

  // Check what a cell contains
  int check(int xcoord, int ycoord)
   {
   	if (xcoord >= 0 && ycoord >= 0 && xcoord < SIZE && ycoord < SIZE)
   	 {
	 	  if(map[xcoord][ycoord] == 0)
           {
           	playfield[xcoord][ycoord]='_'; 
           	map[xcoord][ycoord] = 10;          
            minefree--;
   		    check(xcoord+1, ycoord);
            check(xcoord, ycoord+1);
            check(xcoord-1, ycoord);
            check(xcoord, ycoord-1);
            }
	 	  if(map[xcoord][ycoord] == 1)
	 	   {
	 	   	map[xcoord][ycoord] = 11;
	 	    playfield[xcoord][ycoord]='1';
	 	    minefree--;
	 	   }
	      if(map[xcoord][ycoord] == 2)
	       {
	       	map[xcoord][ycoord] = 12;
	 	    playfield[xcoord][ycoord]='2';
	 	    minefree--;
	       }
	 	  if(map[xcoord][ycoord] == 3)
	 	   {
	 	   	map[xcoord][ycoord] = 13;
	 	    playfield[xcoord][ycoord]='3';
	 	    minefree--;
	 	   }
	 	  if(map[xcoord][ycoord] == 4)
	 	   {
	 	    playfield[xcoord][ycoord]='4';
	 	    map[xcoord][ycoord] = 14;
	 	    minefree--;
	 	   }
	 	  if(map[xcoord][ycoord] == 9)
	 	   {
	 	   	playfield[xcoord][ycoord]='*';
	 	    draw();
	 	    printf("\nGame Over! :(\n");
	 	    exit(0);
	 	   }
	 	  if(minefree==0)
	 	   {
	 	   	draw();
	 	    printf("\nYou Win! :)\n");
	 	    exit(0);
	 	   }
   	 }
    return 1;
   }

int increase()
 {
  //if a mine is nearby we increase the number around by 1
  for (i=0; i < SIZE; i++)
    for (j=0; j < SIZE; j++)
     if (map[i][j]==9)  
      {
      //Left side
       if((j-1)>=0)
        {
       	 //upper left
         if (map[i-1][j-1]!=9) map[i-1][j-1]++;
         //left
         if (map[i][j-1]!=9) map[i][j-1]++;
         //bottom left
         if (map[i+1][j-1]!=9) map[i+1][j-1]++;
        }
       
       //Right side
       if((j+1)<SIZE)
        {
         //upper right
         if (map[i-1][j+1]!=9) map[i-1][j+1]++;
         //right
         if (map[i][j+1]!=9) map[i][j+1]++;
         //bottom right
         if (map[i+1][j+1]!=9) map[i+1][j+1]++; 
        }
       
       //Upper and bottom sides
       //up
       if (map[i-1][j]!=9) map[i-1][j]++;
       //down
       if (map[i+1][j]!=9) map[i+1][j]++;
  }
 }

 // Ask the player for a coordinate
 int coord()
  {
	printf("Insert the X ordinate: ");
	scanf("%d", &x);

	printf("Insert the Y ordinate: ");
	scanf("%d", &y);
	
	if(x>=SIZE || y>=SIZE)
	 {
	  printf("Error! Please insert coordinates between 0 and %d\n", SIZE);
	  coord();
	 }
  }

 int main(int argv, char argc[])
  {
    for(i=0; i<SIZE; i++)
     for(j=0; j<SIZE; j++)
      {
       map[i][j] = 0;
       playfield[i][j]='#';
      }
       
    // Place mines randomly
    srand(time(NULL));
    i=0;
    
    // Absolutely 8 mines per field!
    while (mines!=MAX_MINES)
     {
      if(map[rand() % SIZE + 1][rand() % SIZE + 1]!=9)
       {
        map[rand() % SIZE + 1][rand() % SIZE + 1]=9;
        mines++;
       }
     } 

   increase();

    //check for mine free spaces
    for(i=0; i<SIZE; i++)
     for(j=0; j<SIZE; j++)
      if(map[i][j] !=9)
       minefree++;

       printf("  _/ \\ _______ / \\_  Demin3s!\n"
              " /  __/__   __\\__  \\  \n"
              " \\_/____ \\ / ____\\_/ Insert a number\n"
              "  //    \\___/    \\\\  between 0 and %d\n"
              " | |   o _ _ o   | | Uncover all mine free cells\n"
              " \\_\\___/  V \\___/_/  for win!\n"
              "  < `    | |   ' >     \n"
              " __\\__.  \\^/  .__/__ \n"
              "/   >           <   \\ \n"
              "\\_.  [_I_I_I_I_]  ._/ \n"
              "   \\_/<       >\\_/ \n\n", SIZE);
              
   scanf("%c", NULL);

   while(1)
    {
      draw();
      coord();
      check(x,y);
    }
   return 0;
  }
